<?php
require_once ALI_APPLICATION_PATH . '/models/AbstractModel.php';
class Ali_Model_Tracking extends Ali_Model_AbstractModel
{
    protected $_lookups = array(
        'keyword',
        'url',
        'visitor',
    );

    public function recordClick(array $data)
    {
        global $wpdb;

        $clickData = array();

        foreach ($this->_lookups as $lookup) {
            $clickData[$lookup . '_id'] = $this->lookupCreate($lookup,
                $data[$lookup]);
        }

        $clickData['datetime'] = date('Y-m-d H:i:s');

        $res = $wpdb->insert($this->prefixTable('click'), $clickData);
        if (!$res) {
            throw new Exception("Insert failed: {$wpdb->last_error}");
        }

        return $wpdb->insert_id;
    }

    public function findLastUrlForKeyword($keyword)
    {
        global $wpdb;

        $result = $wpdb->get_var(
            $wpdb->prepare(
                    "SELECT u.`value` FROM `{$this->prefixTable('click')}` c
                    JOIN `{$this->prefixTable('url')}` u ON u.`url_id` = c.`url_id`
                    JOIN `{$this->prefixTable('keyword')}` k ON k.`keyword_id` = c.`keyword_id`
                    WHERE k.`value` = %s
                    ORDER BY c.`datetime` DESC
                    LIMIT 1",
                array(
                    $keyword,
                )
            )
        );

        return $result;
    }

    public function lookup($table, $value, $field='value', $format='%s')
    {
        global $wpdb;

        $result = $wpdb->get_var(
            $wpdb->prepare(
                "SELECT `{$table}_id` FROM `{$this->prefixTable($table)}`
                    WHERE `{$field}` = {$format}",
                array(
                    $value,
                )
            )
        );

        return $result;
    }

    public function lookupCreate($table, $value, $field='value', $format='%s')
    {
        global $wpdb;

        $result = $this->lookup($table, $value, $field);
        if (!$result) {
            $res = $wpdb->insert($this->prefixTable($table),
                array(
                    $field => $value,
                ),
                array(
                    $format
                )
            );
            if (!$res) {
                throw new Exception("Insert failed: {$wpdb->last_error}");
            }

            $result = $wpdb->insert_id;
        }

        return $result;
    }

    public function report($type)
    {
        global $wpdb;

        return $wpdb->get_results(
            "SELECT l.`value`, COUNT(DISTINCT c.`keyword_id`, c.`url_id`, c.`visitor_id`) `unique`, COUNT(*) `raw`
                FROM `{$this->prefixTable('click')}` c
                JOIN `{$this->prefixTable($type)}` l ON c.`{$type}_id` = l.`{$type}_id`
                GROUP BY c.`{$type}_id`
                ORDER BY `unique` DESC"
        );
    }

    public function clear()
    {
        global $wpdb;

        $wpdb->query("TRUNCATE TABLE `{$this->prefixTable('click')}`");
    }

    public function prefixTable($name)
    {
        global $wpdb;

        return $wpdb->prefix . Ali_WpPlugin::PREFIX . '_' . $name;
    }
}